--[[
#Butterfly Model ODD Description 

The butterfly model comes from the work of Pe’er et al. (2005). The description is taken from Section 3.4 of Railsback and Grimm (2012). 

##Purpose

The model was designed to explore questions about virtual corridors. Under what conditions do the interactions of butterfly hilltopping behavior and landscape topography lead to the emergence of virtual corridors, that is, relatively narrow paths along which many butterflies move? How does variability in the butterflies’ tendency to move uphill affect the emergence of virtual corridors?

##Entities, State Variables, and Scales

The model has two kinds of entities: butterflies and square patches of land. The patches make up a square grid landscape of 150 × 150 patches, and each patch has one state variable: its elevation. Butterflies are characterized only by their location, described as the patch they are on. Therefore, butterfly locations are in discrete units, the x- and y- coordinates of the center of their patch. Patch size and the length of one time step in the simulation are not specified because the model is generic, but when real landscapes are used, a patch corresponds to 25 × 25 m<sup>2</sup>. Simulations last for 1000 time steps; the length of one time step is not specified but should be about the time it takes a butterfly to move 25–35 m (the distance from one cell to one of its neighbor cells).

##Process Overview and Scheduling

There is only one process in the model: movement of the butterflies. On each time step, each butterfly moves once. The order in which the butterflies execute this action is unimportant because there are no interactions among the butterflies.

##Design Concepts

The _basic principle_ addressed by this model is the concept of virtual corridors—pathways used by many individuals when there is nothing particularly beneficial about the habitat in them. This concept is addressed by seeing when corridors _emerge_ from two parts of the model: the adaptive movement behavior of butterflies and the landscape they move through. This _adaptive behavior_ is modeled via a simple empirical rule that reproduces the behavior observed in real butterflies: moving uphill. This behavior is based on the understanding (not included in the model) that moving uphill leads to mating, which conveys fitness (success at passing on genes, the presumed ultimate objective of organisms). Because the hilltopping behavior is assumed a priori to be the objective of the butterflies, the concepts of _Objectives_ and _Prediction_ are not explicitly considered. There is no _learning_ in the model.

_Sensing_ is important in this model: butterflies are assumed able to identify which of the surrounding patches has the highest elevation, but to use no information about elevation at further distances. (The field studies of Pe’er 2003 addressed this question of how far butterflies sense elevation differences.)

The model does not include _interaction_ among butterflies; in field studies, Pe’er (2003) found that real butterflies do interact (they sometimes stop to visit each other on the way uphill) but decided it is not important to include interaction in a model of virtual corridors.

_Stochasticity_ is used to represent two sources of variability in movement that are too complex to represent mechanistically. Real butterflies do not always move directly uphill, likely because of (1) limits in the ability of the butterflies to sense the highest area in their neighborhood, and (2) factors other than topography (e.g., flowers that need investigation along the way) that influence movement direction. This variability is represented by assuming butterflies do not move uphill every time step; sometimes they move randomly instead. Whether a butterfly moves directly uphill or randomly at any time step is modeled stochastically, using a parameter _q_ that is the probability of an individual moving directly uphillinstead of randomly.

To allow _observation_ of virtual corridors, we will define a specific “corridor width” measure that characterizes the width of a butterfly’s path from its starting patch to a hilltop.

##Initialization

The topography of the landscape (the elevation of each patch) is initialized when the model starts. Two kinds of landscapes are used in different versions of the model: (1) a simple artificial topography, and (2) the topography of a real study site, imported from a file containing elevation values for each patch. The butterflies are initialized by creating five hundred of them and setting their initial location to a single patch or small region.

##Input Data

The environment is assumed to be constant, so the model has no input data.

##Submodels

The movement submodel defines exactly how butterflies decide whether to move uphill or randomly. First, to “move uphill” is defined specifically as moving to the neighbor patch that has the highest elevation; if two patches have the same elevation, one is chosen randomly. “Move randomly” is defined as moving to one of the neighboring patches, with equal probability of choosing any patch. “Neighbor patches” are the eight patches surrounding the butterfly’s current patch. The decision of whether to move uphill or randomly is controlled by the parameter _q_, which ranges from 0.0 to 1.0 (_q_ is a global variable: all butterflies use the same value). On each time step, each butterfly draws a random number from a uniform distribution between 0.0 and 1.0. If this random number is less than _q_, the butterfly moves uphill; otherwise, the butterfly moves randomly.

## CREDITS AND REFERENCES
Pe’er, G., Saltz, D. & Frank, K. 2005. Virtual corridors for conservation management. _Conservation Biology_, 19, 1997–2003.

Pe’er, G. 2003. Spatial and behavioral determinants of butterfly movement patterns in topographically complex landscapes. Ph.D. thesis, Ben-Gurion University of the Negev.

Railsback, S. & Grimm, V. 2012. _Agent-based and individual-based modeling: A practical introduction_. Princeton University Press, Princeton, NJ.


]]--

tmepath = os.getenv("TME_PATH_1_3_0")
package.path = package.path .. ";"..tmepath.."/contrib/viewerlib/?.lua;"..tmepath.."/contrib/terrameaddons/?.lua"

local rand   = require ("random")
local gris   = require ("grissom")
local view   = require ("viewer")
local util   = require ("util")


Q = 0.7 --threshold for the butterfly's behavior

function createCellSpace (model)
    model.cells = CellularSpace {
        xdim = model.cellSpaceDim,
        ydim = model.cellSpaceDim
    }

    model.cells:createNeighborhood{
        strategy = "moore",
        self = false
    }
    --initalize the two observed variables
    forEachCell (model.cells, function (cell)
        cell.height = 0
        cell.nbutterflies = 0
    end)

    c1 = createHill(100,24)
    insertPattern (model.cells, c1, 25, 25)
    insertPattern (model.cells, c1, 75, 75)
    insertPattern (model.cells, c1, 25, 75)
    insertPattern (model.cells, c1, 75, 25)
    
    -- find the start cell in the cellspace
    model.startCell = model.cells:getCell(Coord{x=model.takeoff.x, y=model.takeoff.y})

end


function insertPattern (cs1, cs2, xc, yc)
    local radiusX = (cs2.xdim - 1)/2
    local radiusY = (cs2.ydim - 1)/2
    assert (xc > radiusX, "insertPattern: cannot insert cell space")
    assert (yc > radiusY, "insertPattern: cannot insert cell space")
    assert (cs1.xdim > (xc + radiusX), "insertPattern: cannot insert cell space")
    assert (cs1.ydim > (yc + radiusY), "insertPattern: cannot insert cell space")
	for i = -radiusY, radiusY do
		for j = -radiusX, radiusX do
            xcell = radiusX + i
            ycell = radiusY + j
			local h = cs2:getCell(Coord{x = xcell, y = ycell}).height
			cs1:getCell(Coord{x=xc + j ,y = yc + i}).height = h
		end
	end
end 

--[[
	creates a hill in the elevation model at a mid point with a certain height
	hill = cone
]]--
function createHill (heightmax, radius)
    local cs = CellularSpace { xdim = 2*radius + 1, ydim = 2*radius + 1 }
    local centre   = radius
    local smooth   = 0.80*radius
    for i = -radius, radius do
        for j = -radius, radius do
            cs:getCell(Coord{x = centre + j, y = centre + i}).height 
                = heightmax*math.exp ( -(j/smooth)^2 -(i/smooth)^2)
        end
    end
    return cs
end
            
function moveUphill(ag)
	agentcell = ag:getCell()
    if agentcell:getNeighborhood():size() > 0 then
        local max = Cell {height = -1.0}
        local up = false
        forEachNeighbor(agentcell, function(cell,neigh)
            if neigh.height > max.height then
                max = neigh 
                up  = true
            end
        end)
        if up then ag:move (max) end
    end		
end

function createAgent (model)
	
	model.butterfly = Agent {
        init =      function (ag)
                        ag.color = rand:integer (3,8)
                    end,
        execute =   function (ag)
                        --draw random number [0,1]
                        decision = rand:integer(0,1)
                        if (decision < Q) then
                            moveUphill(ag)
                        else
                            ag:randomWalk()
                        end
		
                    end
	}
end

function createSociety (model)
    model.society = Society {
        instance = model.butterfly,
        quantity = model.nbutterflies
  }
  forEachAgent (model.society, function (ag) print (ag.color) end)
end

function createEnvironment (model)

    model.env = Environment {model.cells, model.society}

    model.env:createPlacement { strategy = "void", max = model.butterflies }
  
  -- let every agent spawn in this location
    forEachAgent(model.society, function(agent)
                            agent:enter(model.startCell)
                end)
  
end

function createTimer (model)
  model.timer = Timer {
	  Event {time = 1, period = 1, action = function (e)
              model.society:execute()
              --viewAgents(model)
              model.society:notify()
              model.cells:notify()
            end}
  }
end

function createMap (model)
    model.mapcs = view.Map {
            subject    = model.cells,
            attributes = {"height"},
            grouping   = "equalsteps",
            minimum    = 0,
            maximum    = 100,
            slices     = 20,
            colors     = "Yellow-Orange-Brown"
        }
        
    model.sym = view.Symbol {
                subject   = model.society, 
                attributes = {"color"}, 
                observer  = model.mapcs,
                grouping  = "uniquevalue",
                values    = {0,1,2,3,4,5,6,7,8},
                font      = "Grissom Free",
                fontSize  = 24,
                colors    = "Blues",
                symbol    = gris.BUTTERFLY
        }
        
end

function viewAgents (model)
    forEachAgent (model.society, function (ag) ag:notify() end)
end
            

function ButterflyModel (model)

  -- createCellSpace
  createCellSpace(model)

  -- create Agent
  createAgent(model)

  -- create Society
  createSociety(model)

  -- create Environment
  createEnvironment(model)

  -- timer
  createTimer(model)

  -- create Observer
  createMap(model)
  
  return model
end

function run (model, time)
  model.timer:execute (time)
end

model = ButterflyModel {
            cellSpaceDim = 101,
            nbutterflies  = 100,
            takeoff = {
                    x = 50,
                    y = 50
            }
}

run (model, 100)